@@ -69,7 +69,7 @@ mod error;
 pub use arrow;
 pub use odbc_api;
 
-pub use self::{column_strategy::ColumnFailure, error::Error, schema::arrow_schema_from};
+pub use self::{column_strategy::{BufferAllocationOptions, ColumnFailure}, error::Error, schema::arrow_schema_from};
 
 /// Arrow ODBC reader. Implements the [`arrow::record_batch::RecordBatchReader`] trait so it can be
 /// used to fill Arrow arrays from an ODBC data source.
@@ -166,6 +166,46 @@ impl<C: Cursor> OdbcReader<C> {
         max_batch_size: usize,
         schema: SchemaRef,
     ) -> Result<Self, Error> {
+        Self::with(
+            cursor,
+            max_batch_size,
+            Some(schema),
+            BufferAllocationOptions::default(),
+        )
+    }
+
+    /// Construct a new [`crate::OdbcReader`] instance. This method allows you full control over
+    /// what options to explicitly specify, and what options you want to leave to this crate to
+    /// automatically decide.
+    ///
+    /// # Parameters
+    ///
+    /// * `cursor`: ODBC cursor used to fetch batches from the data source. The constructor will
+    ///   bind buffers to this cursor in order to perform bulk fetches from the source. This is
+    ///   usually faster than fetching results row by row as it saves roundtrips to the database.
+    ///   The type of these buffers will be inferred from the arrow schema. Not every arrow type is
+    ///   supported though.
+    /// * `max_batch_size`: Maximum batch size requested from the datasource.
+    /// * `schema`: Arrow schema. Describes the type of the Arrow Arrays in the record batches, but
+    ///    is also used to determine CData type requested from the data source. Set to `None` to
+    ///    infer schema from the data source.
+    /// * `buffer_allocation_options`: Allows you to specify upper limits for binary and / or text
+    ///    buffer types. This is useful support fetching data from e.g. VARCHAR(max) or
+    ///    VARBINARY(max) columns, which otherwise might lead to errors, due to the ODBC driver
+    ///    having a hard time specifying a good upper bound for the largest possible expected value.
+    pub fn with(
+        cursor: C,
+        max_batch_size: usize,
+        schema: Option<SchemaRef>,
+        buffer_allocation_options: BufferAllocationOptions,
+    ) -> Result<Self, Error> {
+        // Infer schema if not given by the user
+        let schema = if let Some(schema) = schema {
+            schema
+        } else {
+            Arc::new(arrow_schema_from(&cursor)?)
+        };
+
         let column_strategies: Vec<Box<dyn ColumnStrategy>> = schema
             .fields()
             .iter()
@@ -174,15 +214,47 @@ impl<C: Cursor> OdbcReader<C> {
                 let col_index = (index + 1).try_into().unwrap();
                 let lazy_sql_data_type = || cursor.col_data_type(col_index);
                 let lazy_display_size = || cursor.col_display_size(col_index);
-                choose_column_strategy(field, lazy_sql_data_type, lazy_display_size)
-                    .map_err(|cause| cause.into_crate_error(field.name().clone(), index))
+                choose_column_strategy(
+                    field,
+                    lazy_sql_data_type,
+                    lazy_display_size,
+                    buffer_allocation_options,
+                )
+                .map_err(|cause| cause.into_crate_error(field.name().clone(), index))
             })
             .collect::<Result<_, _>>()?;
 
         let row_set_buffer = buffer_from_description(
             max_batch_size,
             column_strategies.iter().map(|cs| cs.buffer_description()),
-        );
+        )
+        .map_err(|source| match source {
+            odbc_api::Error::FailedSettingConnectionPooling
+            | odbc_api::Error::FailedAllocatingEnvironment
+            | odbc_api::Error::NoDiagnostics { .. }
+            | odbc_api::Error::Diagnostics { .. }
+            | odbc_api::Error::AbortedConnectionStringCompletion
+            | odbc_api::Error::UnsupportedOdbcApiVersion(_)
+            | odbc_api::Error::FailedReadingInput(_)
+            | odbc_api::Error::InvalidRowArraySize { .. }
+            | odbc_api::Error::OracleOdbcDriverDoesNotSupport64Bit(_)
+            | odbc_api::Error::TooLargeValueForBuffer
+            | odbc_api::Error::TooManyDiagnostics => {
+                panic!("Unexpected error in upstream ODBC api error library")
+            }
+            odbc_api::Error::TooLargeColumnBufferSize {
+                buffer_index,
+                num_elements,
+                element_size,
+            } => Error::ColumnFailure {
+                name: schema.field(buffer_index as usize).name().clone(),
+                index: buffer_index as usize,
+                source: ColumnFailure::TooLarge {
+                    num_elements,
+                    element_size,
+                },
+            },
+        })?;
         let cursor = cursor.bind_buffer(row_set_buffer).unwrap();
 
         Ok(Self {
@@ -200,7 +272,7 @@ where
     type Item = Result<RecordBatch, ArrowError>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        match self.cursor.fetch() {
+        match self.cursor.fetch_with_truncation_check(true) {
             // We successfully fetched a batch from the database. Try to copy it into a record batch
             // and forward errors if any.
             Ok(Some(batch)) => {
